/* Digital Protocol Dialogue */
#include "stdafx.h"
#include "math.h"

const int PRO_ITER_LEN = 6;
const int PRO_DUR_LEN = 7;
const int PRO_LINES_LEN = 2;
const int PRO_TIMECOURSE_LEN = 128;

class CVwDigitalProtocol : public CDialog { 
	public : 
		CDigitalProtocol m_oDigPro;			// Underlying object for Protocol data 
		RECT m_rctDisp;					// Boundary for Disp protocol window 
		CString m_cLongName;			// file name - blank on New 

		CDlgEdit m_edtDuration;
		CDlgEdit m_edtIterations;
		CDlgEdit m_edtLines;
		CDlgEdit m_edtTimeCourse[MAXDOLINES];
		
		CDlgControl m_picDisp;

		int m_aiCapture[5];		// The panel, x, y when mouse capture for sizing panels begins

		CVwDigitalProtocol() { 
			m_iID = IDD_DIGITALPROTOCOL;
			m_cLongName.Set("");
			m_pCapture=NULL;
			m_aiCapture[0]=NOCAPTURE;
			}

		void NewProtocol() {			// Build a simple default protocol for new ones
			// all handled in constructor
			}

		void InitializeControls( ) { 
			m_edtDuration.SetHandle( GetDlgItem( m_hDlg, IDC_PRO_DUR ), "Total duration per iteration in milliseconds" );
			m_edtDuration.SetTextLimit( PRO_DUR_LEN );

			m_edtIterations.SetHandle( GetDlgItem( m_hDlg, IDC_PRO_ITER ), "Number of times to generate protocol" );
			m_edtIterations.SetTextLimit( PRO_ITER_LEN );

			m_edtLines.SetHandle( GetDlgItem( m_hDlg, IDC_PRO_LINES ), "Number of digital output lines to address" );
			m_edtLines.SetTextLimit( PRO_LINES_LEN );
			
			for( int iLine=0; iLine<MAXDOLINES; ++iLine ) {
				m_edtTimeCourse[iLine].SetHandle( GetDlgItem( m_hDlg, IDC_TIMECOURSE0+iLine ), "List of off,on,off,on times in milleseconds.  Example(1 sec off, 2 sec on, off ) : 1000,2000,1" );
				m_edtTimeCourse[iLine].SetTextLimit( PRO_TIMECOURSE_LEN );
				}
			
			m_picDisp.SetHandle( GetDlgItem( m_hDlg, IDC_TIMECOURSES ) );
			m_picDisp.GetControlBounds( &m_rctDisp );
			CreateTip( m_rctDisp.left, m_rctDisp.top, m_rctDisp.right, m_rctDisp.bottom, "Drag in empty time course to create pulse." );

			}

		void CauseRedraw( ) { 			
			m_rctDisp.top-=10;
			InvalidateRect( m_hDlg, &m_rctDisp, true );
			m_rctDisp.top+=10;
			}

		bool SetupProtocol( ) { 
			//-- Duration Total
			SetDlgItemInt( m_hDlg, IDC_PRO_DUR, m_oDigPro.m_iDuration, FALSE );
			
			//-- Iterations
			SetDlgItemInt( m_hDlg, IDC_PRO_ITER, m_oDigPro.m_iIterations, FALSE );

			//-- Lines
			SetDlgItemInt( m_hDlg, IDC_PRO_LINES, m_oDigPro.m_iLines, FALSE );

			//-- Time Course text 
			for( int iLine=0; iLine<MAXDOLINES; ++iLine ) {
				Parse( iLine );
				SetDlgItemText( m_hDlg, IDC_TIMECOURSE0+iLine, m_oDigPro.m_acTimeString[iLine].Buffer());
				}
			
			return true;
			}
			
		void EnableTimeCourses() { 
			for( int iLine=0;iLine<MAXDOLINES; ++iLine ) 
				m_edtTimeCourse[iLine].SetEnabled( iLine<m_oDigPro.m_iLines );
			}

		void Parse( int iTimeCourse ) { 
			m_oDigPro.m_aTimeCourse[iTimeCourse].Parse( m_oDigPro.m_acTimeString[iTimeCourse].Buffer() );
			}
			
		void DrawProtocol(HDC hdc) { 
			HBRUSH brRed = CreateSolidBrush( RGB(255,0,0) );
			HBRUSH brGreen = CreateSolidBrush(RGB(0,255,0) );
			HBRUSH brLtRed = CreateSolidBrush(RGB(128,0,0) );
			HBRUSH brLtGreen = CreateSolidBrush(RGB(0,128,0) );

			//-- Time conversion 
			int iPicWid = m_rctDisp.right-m_rctDisp.left;

			//-- Draw tickmarks
			HPEN penGray = CreatePen( 0, 0, RGB(64,64,64) );
			HPEN penWas = (HPEN) SelectObject( hdc, penGray );
			for( int i=0; i<m_oDigPro.m_iDuration; i+=1000 ) { 
				int iTickPos = MulDiv( i, iPicWid, m_oDigPro.m_iDuration );
				MoveToEx( hdc, m_rctDisp.left+iTickPos, m_rctDisp.top-10, NULL );
				LineTo( hdc, m_rctDisp.left+iTickPos, m_rctDisp.top );
				}
			SelectObject( hdc, penWas );
			DeleteObject( penGray );
							
			//-- Draw timecourses 
			RECT r;
			for( int iLine=0; iLine<MAXDOLINES; ++iLine ) { 
				CDigitalTimeCourse * pTC = &(m_oDigPro.m_aTimeCourse[iLine]);
				r.top = (m_rctDisp.top*(MAXDOLINES-iLine)+m_rctDisp.bottom*iLine)/MAXDOLINES;
				r.bottom = (m_rctDisp.top*(MAXDOLINES-iLine-1)+m_rctDisp.bottom*(iLine+1))/MAXDOLINES-1;
				r.left=r.right=m_rctDisp.left;

				//-- draw alternating red/green blocks for on/off
				int iTime = 0; 
				for( int iTC=0; iTC<pTC->m_iSegCount; ++iTC ) { 
					r.left = min( m_rctDisp.right, MulDiv( iTime, iPicWid, m_oDigPro.m_iDuration ) + m_rctDisp.left );
					r.right = min( m_rctDisp.right, MulDiv( iTime+pTC->m_aiTimes[iTC], iPicWid, m_oDigPro.m_iDuration ) + m_rctDisp.left );
					FillRect( hdc, &r, iTC%2==0?brRed:brGreen );
					iTime += pTC->m_aiTimes[iTC];
					}
				//-- draw a light red or green block to end of protocol to show end state	
				if( r.right < m_rctDisp.right ) {
					r.left = r.right+1;
					r.right = m_rctDisp.right;
					FillRect( hdc, &r, (pTC->m_iSegCount==0) || (pTC->m_iSegCount%2==1)?brLtRed:brLtGreen );
					}
				}

			//-- Cleanup 
			DeleteObject( brLtGreen );
			DeleteObject( brLtRed );
			DeleteObject( brGreen );
			DeleteObject( brRed );

			//-- extra while dragging 			
			if( m_aiCapture[0]!=NOCAPTURE ) { 
				//MoveToEx( hdc, m_aiCapture[1], m_aiCapture[2], NULL );
				//LineTo( hdc, m_aiCapture[3], m_aiCapture[4] );
				}
			}				
			
		//---------------------------------------------------------------------
		void DragInvalidate() { 
			RECT rct;
			SetRect( &rct, m_rctDisp.left, min(m_aiCapture[2],m_aiCapture[4]), 
					m_rctDisp.right, max(m_aiCapture[2],m_aiCapture[4])+1 );
			InvalidateRect( m_hDlg, &rct, false );
			}
			
		bool DragDown( int iX, int iY ){ //TODO - mouse parameters should be passed as short
			//-- click on draggable control item
			if( CDialog::DragDown( iX, iY ) )
				return true;
				
			//-- click in trace to toggle
			if( iX>m_rctDisp.left && iX<m_rctDisp.right
					&& iY>=m_rctDisp.top && iY<m_rctDisp.bottom ) {
				m_aiCapture[0]=1;
				m_aiCapture[1]=iX;
				m_aiCapture[2]=iY;
				m_aiCapture[3]=iX;
				m_aiCapture[4]=iY;
				SetCapture( m_hDlg );
				DragInvalidate();
				return true;
				}
			return false;
			}
			
		void DragMove( int iX, int iY ){
			if( m_aiCapture[0]!=NOCAPTURE ) {
				DragInvalidate();
				m_aiCapture[3]=iX;
				m_aiCapture[4]=iY;
				DragInvalidate();
				return;
				}
				
			CDialog :: DragMove( iX, iY );
			}

		void DragUp( int iX, int iY ){
			if( m_aiCapture[0]!=NOCAPTURE ) {
				ReleaseCapture();
				DragInvalidate();
				m_aiCapture[3]=iX;
				m_aiCapture[4]=iY;
				DragInvalidate();

				//-- Apply adjustment to trace 
				int iLine = max(0,min(MAXDOLINES-1, (m_aiCapture[2]-m_rctDisp.top)*MAXDOLINES/(m_rctDisp.bottom-m_rctDisp.top) ));
				int iPicWid = m_rctDisp.right-m_rctDisp.left;
				int iT0 = MulDiv( m_oDigPro.m_iDuration, m_aiCapture[1]-m_rctDisp.left, iPicWid );
				int iT1 = MulDiv( m_oDigPro.m_iDuration, iX-m_rctDisp.left, iPicWid );
				m_oDigPro.Inject( iLine, 250*int((iT0+125)/250) );
				m_oDigPro.Inject( iLine, 250*int((iT1+125)/250) );
				m_edtTimeCourse[iLine].SetText(m_oDigPro.m_acTimeString[iLine].ConstBuffer());
				
				//--
				m_aiCapture[0]=NOCAPTURE; 
				InvalidateRect( m_hDlg, &m_rctDisp, false );
				return;
				}

			CDialog :: DragUp( iX,iY );
			}
			
			
	};

//-- Protocol view message handler
BOOL CALLBACK DigitalProtocolDlgProc(HWND hDlg, UINT iMsg, WPARAM wParam, LPARAM lParam) { 
	PAINTSTRUCT ps;
	int wmId, wmEvent;
	CVwDigitalProtocol * pVwPro;
	
	switch( iMsg ) { 
		case WM_INITDIALOG :
			pVwPro = new CVwDigitalProtocol;
			pVwPro->m_hDlg=hDlg;
			SetWindowLong( GetParent( hDlg ), 0,  (long) pVwPro ) ;
			pVwPro->InitializeControls();
			return TRUE;
			
		case WM_PRO_NEW :		// user message for menu item to create a new protocol
			pVwPro = (CVwDigitalProtocol *) GetWindowLong(  GetParent( hDlg ), 0 );
			pVwPro->NewProtocol();		
			pVwPro->SetupProtocol();
			return TRUE;

		case WM_PRO_LOAD :		// User message for menu item to load file 
			pVwPro = (CVwDigitalProtocol *) GetWindowLong(  GetParent( hDlg ), 0 );
			if ( ! pVwPro->m_oDigPro.LoadFile((char *)lParam) ) {
				MessageBox( hDlg, "Error loading digital protocol file", (char *)lParam, MB_OK);
				DestroyWindow( GetParent( hDlg ) );
				return TRUE;
				}
			pVwPro->m_cLongName.Set( (char *)lParam );
			pVwPro->SetupProtocol();
			return TRUE;

		case WM_NOTIFY :		// 
			break;
			
		case WM_COMMAND : {
			wmId    = LOWORD(wParam); 
			wmEvent = HIWORD(wParam);
			pVwPro = (CVwDigitalProtocol *) GetWindowLong(  GetParent( hDlg ), 0 );
		
			if( wmEvent==EN_CHANGE ) { 
				//----- Duration changed
				if( wmId == IDC_PRO_DUR ) {
					pVwPro->m_oDigPro.m_iDuration = pVwPro->m_edtDuration.GetTextInt();
					pVwPro->CauseRedraw();
					return TRUE;
					}
				//----- Iterations changed
				if( wmId == IDC_PRO_ITER ) {
					pVwPro->m_oDigPro.m_iIterations = pVwPro->m_edtIterations.GetTextInt();
					pVwPro->CauseRedraw();
					return TRUE;
					}
				//----- Lines changed
				if( wmId == IDC_PRO_LINES ) {
					pVwPro->m_oDigPro.m_iLines = pVwPro->m_edtLines.GetTextInt();
					pVwPro->EnableTimeCourses();
					pVwPro->CauseRedraw();
					return TRUE;
					}
			
				//----- a TimeCourse text changed
				if( wmId>=IDC_TIMECOURSE0 && wmId<=IDC_TIMECOURSE16 ) { 
					int iTC = wmId-IDC_TIMECOURSE0;
					pVwPro->m_edtTimeCourse[iTC].GetText(&(pVwPro->m_oDigPro.m_acTimeString[iTC]));
					pVwPro->Parse( iTC );
					pVwPro->CauseRedraw();
					return TRUE;
					}
				}

			//----- Save Protocol
			if( wmId == IDC_BTNSAVE ) { 
				char bufName[_MAX_PATH+1]="";
				if( pVwPro->m_cLongName.Length()!=0 && pVwPro->m_cLongName.Length()<_MAX_PATH ) 
					sprintf( bufName, "%s", pVwPro->m_cLongName.ConstBuffer() );
					
				//-- select the file to save
					if( ! GetSaveFileName(bufName, 
									"Protocol Files (*.qdp)\0*.qdp\0All Files (*.*)\0*.*\0\0",
									"qdp", 
									GetParent(hDlg), NULL,
									& g_oCfgs.m_oSettings.m_cProtoPath ) )
						return TRUE;
				pVwPro->m_cLongName.Set(bufName);

				if( pVwPro->m_oDigPro.SaveFile( pVwPro->m_cLongName.Buffer() ) ) 
					DestroyWindow( GetParent( hDlg ) );
				return TRUE;
				}

			//----- Cancel protocol edit 
			if( wmId == IDC_BTNCANCEL ) { 
				DestroyWindow( GetParent( hDlg ) );
				return TRUE;
				}
			}
			return FALSE;
			
		case WM_PAINT : {
			HDC hdc = BeginPaint(hDlg, &ps);

			pVwPro = (CVwDigitalProtocol *) GetWindowLong(  GetParent( hDlg ), 0 );
			pVwPro->DrawProtocol( hdc );
			
			EndPaint(hDlg, &ps);
			}
			return false;
			
		case WM_DESTROY :
			pVwPro = (CVwDigitalProtocol *) GetWindowLong(  GetParent( hDlg ), 0 );
			delete pVwPro;
			SetWindowLong( GetParent( hDlg ), 0, 0 );
			WindowListRemove( hDlg );
			return TRUE;

		//----- 
		case WM_LBUTTONDOWN :		//-- Start of drag operation			
			pVwPro = (CVwDigitalProtocol *) GetWindowLong(  GetParent( hDlg ), 0 );
			if( pVwPro->DragDown( LOWORD(lParam), HIWORD(lParam) ) )
				return TRUE;
			return FALSE;

		case WM_MOUSEMOVE :			//-- Mouse move during drag operation
			pVwPro = (CVwDigitalProtocol *) GetWindowLong(  GetParent( hDlg ), 0 );
			if( pVwPro->m_aiCapture[0]==NOCAPTURE)
				return FALSE;
			pVwPro->DragMove( short(LOWORD(lParam)), short(HIWORD(lParam)) );
			return TRUE;

		case WM_LBUTTONUP :			//-- end of drag operation
			pVwPro = (CVwDigitalProtocol *) GetWindowLong(  GetParent( hDlg ), 0 );
			if( pVwPro->m_pCapture == NULL && pVwPro->m_aiCapture[0]==NOCAPTURE)
				return FALSE;
			pVwPro->DragUp( LOWORD(lParam), HIWORD(lParam) );
			return TRUE;

		default : 
			break;
		}
		
	return FALSE;
	} 

